# encoding: utf-8
class Gw::Admin::Webmail::AddressesController < Gw::Controller::Admin::Base
  include Sys::Controller::Scaffold::Base
  layout "admin/gw/webmail"
  before_action :set_title
  helper Gw::AddressHelper
  
  before_action :set_before_url, only: [:new, :edit]
  
  def set_before_url
    before_path = Rails.application.routes.recognize_path(request.referrer)
    if ["index", "show"].include?(before_path[:action])
      session[:request_referrer] = request.referrer
    end
  end

  def pre_dispatch
    #return error_auth unless Core.user.has_auth?(:designer)
  end

  def index
    return render(text: '')
    item = Gw::WebmailAddress.readable
    item = item.where(group_id: @parent.id) if @parent
    item = item.where(user_id: Core.user.id)
    item = item.order(kana: params[:sort], id: params[:sort]) if params[:sort].present?
    item = item.paginate(page: params[:page], per_page: params[:limit])
    @items = item.all
    _index @items
  end

  def show
    @item = Gw::WebmailAddress.find(params[:id])
    return error_auth unless @item.readable?
    @item.in_groups = @item.groups.collect {|g| g.id}.join(",")
    _show @item
  end

  def new
    @item = Gw::WebmailAddress.new()
  end

  def create
    @item = Gw::WebmailAddress.new(create_address_params)
    @item.user_id = Core.user.id
    _create(@item, location: gw_webmail_address_groups_path)
  end

  def update
    @item = Gw::WebmailAddress.find(params[:id])
    return error_auth unless @item.editable?
    @item.attributes = address_update_params
    @item.user_id = Core.user.id

    _update(@item, location: gw_webmail_address_groups_path)
  end

  def destroy
    @item = Gw::WebmailAddress.find(params[:id])
    return error_auth unless @item.deletable?

    _destroy(@item, :location => gw_webmail_address_groups_path)
  end

  def create_mail
    @item = Gw::WebmailAddress.find(params[:id])
    return error_auth unless @item.readable?
    flash[:mail_to] = "#{@item.name} <#{@item.email}>"
    redirect_to new_gw_webmail_mail_path('INBOX')
  end

  # === CSVインポート画面 アクション
  #
  # ==== 引数
  #  * なし
  # ==== 戻り値
  #  なし
  def import
    @csv_form = RumiHelper::CsvForm.new_import_mode
  end

  # === CSVインポート確認画面 アクション
  #
  # ==== 引数
  #  * なし
  # ==== 戻り値
  #  なし
  def candidate_import
    @csv_form = RumiHelper::CsvForm.new(params[:rumi_helper_csv_form] || {})

    begin
      raise @csv_form.error_full_messages if @csv_form.invalid?

      values = Gw::WebmailAddress.candidate_import_csv(@csv_form)

      @valid_addresses = values[:valid_addresses]
      @invalid_addresses = values[:invalid_addresses]
      @new_groups = values[:new_groups]

      respond_to do |format|
        format.html { render action: :candidate_import }
      end

    rescue => e
      flash.now[:notice] = I18n.t("rumi.private_address_book.action.csv.import.message.error")

      # 本番のログでも出力する
      Rails.logger.error "[ERROR] private address candidate_import_csv Invalid Error"
      Rails.logger.error e.inspect
      Rails.logger.error e.backtrace.join("\n") if e.respond_to?(:backtrace)

      respond_to do |format|
        format.html { render action: :import }
      end
    end
  end

  # === CSVインポート実行 アクション
  #
  # ==== 引数
  #  * なし
  # ==== 戻り値
  #  なし
  def exec_import
    candidate_data = params[:candidate_data] || {}
    # CSV形式を特定するためインスタンスを生成
    csv_form = RumiHelper::CsvForm.new(format_type: params[:csv_format_type])

    begin
      Gw::WebmailAddress.import_csv(candidate_data, csv_form)

      flash[:notice] = I18n.t("rumi.private_address_book.action.csv.import.message.success")
    rescue => e
      # 本番のログでも出力する
      Rails.logger.error "[ERROR] public address exec_import_csv Invalid Error"
      Rails.logger.error e.inspect
      Rails.logger.error e.backtrace.join("\n") if e.respond_to?(:backtrace)

      flash[:notice] = I18n.t("rumi.private_address_book.action.csv.import.message.error")
    ensure
      respond_to do |format|
        format.html { redirect_to action: :import }
      end
    end

  end

  # === CSVエクスポート画面 アクション
  #
  # ==== 引数
  #  * なし
  # ==== 戻り値
  #  なし
  def export
    @csv_form = RumiHelper::CsvForm.new_export_mode
  end

  # === CSVエクスポート実行 アクション
  #
  # ==== 引数
  #  * なし
  # ==== 戻り値
  #  なし
  def exec_export
    @csv_form = RumiHelper::CsvForm.new(params[:rumi_helper_csv_form] || {})

    begin
      raise @csv_form.error_full_messages if @csv_form.invalid?

      csv_string = Gw::WebmailAddress.to_csv(@csv_form)
      filename = Gw::WebmailAddress.export_csv_filename(@csv_form)

      send_data csv_string, filename: filename, type: "text/csv", disposition: "attachment"

    rescue => e
      flash.now[:notice] = I18n.t("rumi.private_address_book.action.csv.export.message.error")

      # 本番のログでも出力する
      Rails.logger.error "[ERROR] private address exec_export_csv Invalid Error"
      Rails.logger.error e.inspect
      Rails.logger.error e.backtrace.join("\n") if e.respond_to?(:backtrace)

      respond_to do |format|
        format.html { render action: :export }
      end
    end
  end
private
  #ヘッダのタイトルを設定するメソッド
    def set_title
      @title = I18n.t('rumi.mail.title.private_address')
    end

  def create_address_params
    params.require(:item).permit(:name, :kana, :sort_no, :in_groups, :email, :mobile_tel, :uri, :tel, :fax,
                                 :zip_code, :address, :company_name, :company_kana, :official_position,
                                 :company_tel, :company_fax, :company_zip_code, :company_address, :memo,
                                 :easy_entry, :escaped, :in_groups)
  end

  def address_update_params
    params.require(:item).permit(:name, :kana, :sort_no, :in_groups, :email, :mobile_tel, :uri, :tel, :fax,
                                 :zip_code, :address, :company_name, :company_kana, :official_position,
                                 :company_tel, :company_fax, :company_zip_code, :company_address, :memo,:in_groups)
  end
protected

  def return_path(item)
    item.group_id ? gw_webmail_address_group_path(item.group_id) : gw_webmail_address_groups_path
  end

  def conv_lf(s)
    s ? s.gsub("\n", '') : nil
  end

  def split_addr(addr)
    addr = addr.to_s
    if I18n.default_locale.to_s == 'ja'
      match = addr.scan(/(.+[都|道|府|県])(.+[市|区|町|村])(.*)/)
    else
      match = []
    end
    return match[0] if match.length >= 1 && match[0].length == 3
    return [addr, '', '']
  end

  def check_duplication_email(email)
    check = Gw::WebmailAddress.select("*").where(:email => email, :user_id => Core.current_user.id)
    if check.size == 0
      return true
    else
      return false
    end
  end
end
