# encoding: utf-8
class Gw::Admin::Webmail::TemplatesController < Gw::Controller::Admin::Base
  include Sys::Controller::Scaffold::Base
  layout "admin/gw/webmail"
  before_action :set_title
  before_action :set_before_url, only: [:new, :edit]
  
  def set_before_url
    before_path = Rails.application.routes.recognize_path(request.referrer)
    if ["index", "show"].include?(before_path[:action])
      session[:request_referrer] = request.referrer
    end
  end

  def pre_dispatch
    # アドレス帳検索 検索結果最大表示件数を取得
    @search_limit = Enisys.config.application['webmail.search_address_max_count'].to_i
  end

  def index
    item = Gw::WebmailTemplate.readable
    item = item.where(user_id: Core.user.id)
               .paginate(page: params[:page], per_page: params[:limit])
               .order(name: :asc, id: :asc)
    @items = item.all
    _index @items
  end

  def show
    @item = Gw::WebmailTemplate.find(params[:id])
    return error_auth unless @item.readable?
    _show @item
  end

  def new
    @item = Gw::WebmailTemplate.new({
      default_flag: 0
    })
  end

  def create
    @item = Gw::WebmailTemplate.new(item_params)
    @item.user_id = Core.user.id
    _create(@item)
  end

  def update
    @item = Gw::WebmailTemplate.find(params[:id])
    return error_auth unless @item.editable?
    @item.attributes = item_params
    @item.user_id = Core.user.id

    _update(@item)
  end

  def destroy
    @item = Gw::WebmailTemplate.find(params[:id])
    return error_auth unless @item.deletable?
    _destroy(@item)
  end
private
  #ヘッダのタイトルを設定するメソッド
  def set_title
    @title = I18n.t('rumi.mail.title.template')
  end

  def item_params
    params.require(:item).permit(:name, :to, :cc, :bcc, :subject, :body, :default_flag)
  end

end
