# encoding: utf-8
class Sys::Admin::AddressesController < Sys::Controller::Admin::Base
  include Sys::Controller::Scaffold::Base
  before_action :set_title
  before_action :set_before_url, only: [:index]
  
  def set_before_url
    before_path = Rails.application.routes.recognize_path(request.referrer)
    if ["index", "show"].include?(before_path[:action]) && !["sys/admin/addresses"].include?(before_path[:controller])
      session[:request_referrer] = request.referrer
    end
  end
  
  def pre_dispatch
    return error_auth unless Core.user.has_auth?(:manager)
  end
  
  def index
    @items = Sys::AddressSetting.all.order(:sort_no)
    @use_readonly_keys = ["name", "email", "state"]
    @list_readonly_keys = ["name", "email"]
    @view_names = I18n.t("activerecord.attributes.gw/webmail_address").stringify_keys
  end

  def create
    # ドラッグ＆ドロップによる並び順変更(Ajax)の場合は更新を行わない。
    return unless item_params
    address_settings = Sys::AddressSetting.all
    begin
      before_sort_no = 0
      item_params.each do |id, item|
        # 取得順が並び順(sort_no)
        before_sort_no += 1
        item["sort_no"] = before_sort_no
        update = address_settings.find(id)
        update.attributes = item
        update.save
      end
      flash[:notice] = I18n.t("rumi.message.address_update")
    rescue
      flash[:error] = I18n.t("rumi.message.fail_address_update")
    end
    return redirect_to(:action => :index)
  end
private
  #ヘッダのタイトルを設定するメソッド
  def set_title
    @title = I18n.t('rumi.mail.title.setting.admin.address')
  end

  def item_params
    params.require(:item).permit!
  end
end
