# encoding: utf-8
class Sys::Admin::AttackMailExtensionsController < Sys::Controller::Admin::Base
  include Sys::Controller::Scaffold::Base
  before_action :set_title
  before_action :set_before_url, only: [:new, :edit]
  
  def set_before_url
    before_path = Rails.application.routes.recognize_path(request.referrer)
    if ["index", "show"].include?(before_path[:action])
      session[:request_referrer] = request.referrer
    end
  end

  def pre_dispatch
    return error_auth unless Core.user.has_auth?(:manager)
  end
  
  def index
    @items = Sys::AttackMailExtension.all
  end

  def show
    @item = Sys::AttackMailExtension.find(params[:id])
  end

  def new
    @item = Sys::AttackMailExtension.new
  end

  def edit
    @item = Sys::AttackMailExtension.find(params[:id])
  end

  def create
    @item = Sys::AttackMailExtension.new
    @item.attributes = item_params
    if @item.save
      flash[:notice] = I18n.t("helpers.message.regist")
      redirect_to sys_attack_mail_extensions_path
    else
      flash.now[:notice] = I18n.t("helpers.message.fail_regist")
      respond_to do |format|
        format.html { render :action => "new" }
        format.xml  { render :xml => @item.errors, :status => :unprocessable_entity }
      end
    end
  end

  def update
    @item = Sys::AttackMailExtension.find(params[:id])
    @item.attributes = item_params
    if @item.save
      flash[:notice] = I18n.t("helpers.message.update")
      redirect_to sys_attack_mail_extensions_path
    else
      flash.now[:notice] = I18n.t("helpers.message.fail_update")
      respond_to do |format|
        format.html { render :action => "edit" }
        format.xml  { render :xml => @item.errors, :status => :unprocessable_entity }
      end
    end
  end

  def destroy
    @item = Sys::AttackMailExtension.find(params[:id])
    if @item.deletable? && @item.destroy
      flash[:notice] = I18n.t("helpers.message.delete")
      redirect_to sys_attack_mail_extensions_path
    else
      flash.now[:notice] = I18n.t("helpers.message.fail_delete")
      respond_to do |format|
        format.html { render :action => :index }
        format.xml  { render :xml => @item.errors, :status => :unprocessable_entity }
      end
    end
  end
private
  #ヘッダのタイトルを設定するメソッド
  def set_title
    @title = I18n.t('rumi.mail.title.setting.admin.attack_mail')
  end

  def item_params
    params.require(:item).permit(:extension)
  end
end
