# encoding: utf-8
class Sys::Admin::AttackMailMessagesController < Sys::Controller::Admin::Base
  include Sys::Controller::Scaffold::Base
  before_action :set_title
  before_action :set_before_url, only: [:new, :edit]
  
  def set_before_url
    before_path = Rails.application.routes.recognize_path(request.referrer)
    if ["index", "show"].include?(before_path[:action])
      session[:request_referrer] = request.referrer
    end
  end
  
  def pre_dispatch
    return error_auth unless Core.user.has_auth?(:manager)
  end
  
  def index
    @item = Sys::AttackMailMessage.first
    if @item.blank?
      @item = Sys::AttackMailMessage.new
      @item.id = 1
      @item.title = t("rumi.sys.message.table.default_title")
      @item.common = t("rumi.sys.message.table.default_common")
      @item.domain = t("rumi.sys.message.table.default_domain")
      @item.extension = t("rumi.sys.message.table.default_extension")
      @item.information = t("rumi.sys.message.table.default_information")
      @item.save
    end
  end

  def edit
    @item = Sys::AttackMailMessage.where(id: params[:id]).first
  end

  def update
    @item = Sys::AttackMailMessage.where(id: params[:id]).first
    @item.attributes = item_params
    if @item.save
      flash[:notice] = I18n.t("helpers.message.update")
      redirect_to sys_attack_mail_messages_path
    else
      flash.now[:notice] = I18n.t("helpers.message.fail_update")
      respond_to do |format|
        format.html { render :action => "edit" }
        format.xml  { render :xml => @item.errors, :status => :unprocessable_entity }
      end
    end
  end
private
  #ヘッダのタイトルを設定するメソッド
  def set_title
    @title = I18n.t('rumi.mail.title.setting.admin.attack_mail')
  end

  def item_params
    params.require(:item).permit(:title, :common, :domain, :extension, :information)
  end
end
