# encoding: utf-8
class Sys::Admin::LdapSynchrosController < Sys::Controller::Admin::Base
  include Sys::Controller::Scaffold::Base

  def pre_dispatch
    return error_auth unless Core.user.has_auth?(:manager)
    return render(:text=> I18n.t("rumi.ldap.message.fail_connect"), :layout => true) unless Core.ldap.connection
  end

  def index
    item = Sys::LdapSynchro.new#.readable
    item.page  params[:page], params[:limit]
    item.order params[:sort], 'version DESC'
    @items = item.group(:version)
    _index @items
  end

  def show
    @version = params[:id]

    item = Sys::LdapSynchro.new
    item.and :version, @version
    item.and :parent_id, 0
    item.and :entry_type, 'group'
    @items = item.order('sort_no, code')

    _show @items
  end

  def new
    @item = Sys::LdapSynchro.new({})
  end

  def create
    @version = Time.now.strftime('%s')
    @results = {:group => 0, :user => 0, :error => 0}
    error   = nil

    begin
      create_synchros
    rescue ActiveLdap::LdapError::AdminlimitExceeded
      error = I18n.t("rumi.ldap.time_over")
    rescue
      error = ""
    end

    if error.nil?
      messages = [I18n.t("rumi.ldap.message.middle_data")]
      messages << "-- " + I18n.t("rumi.attributes.group") + "#{@results[:group]}" + I18n.t("helpers.number.unit.addresses")
      messages << "-- "+ I18n.t("rumi.attributes.user") +  "#{@results[:user]}" + I18n.t("helpers.number.unit.addresses")
      messages << "-- "+ I18n.t("rumi.attributes.error") +  "#{@results[:error]}" + I18n.t("helpers.number.unit.addresses") if @results[:error] > 0
      flash[:notice] = messages.join('<br />').html_safe
      redirect_to url_for(:action => :show, :id => @version)
    else
      flash[:notice] = I18n.t("rumi.ldap.message.fail_middle_data") + "［ #{error} ］"
      redirect_to url_for(:action => :index)
    end
  end

  def update

  end

  def destroy
    Sys::LdapSynchro.where('version = ?', params[:id]).delete_all
    flash[:notice] = I18n.t("helpers.message.delete")
    redirect_to url_for(:action => :index)
  end

  def synchronize
    @version = params[:id]

    item = Sys::LdapSynchro.new
    item.and :version, @version
    item.and :parent_id, 0
    item.and :entry_type, 'group'
    @items = item.order('sort_no, code')

    unless parent = Sys::Group.find_by(parent_id: 0)
      return render :inline => I18n.t("rumi.ldap.message.root_group"), :layout => true
    end

    Sys::Group.update_all("ldap_version = NULL")
    Sys::User.update_all("ldap_version = NULL")

    @results = {:group => 0, :gerr => 0, :user => 0, :uerr => 0}
    @items.each {|group| do_synchro(group, parent)}

    @results[:udel] = Sys::User.where("ldap = 1 AND ldap_version IS NULL").destroy_all.size
    @results[:gdel] = Sys::Group.where("parent_id != 0 AND ldap = 1 AND ldap_version IS NULL").destroy_all.size

    messages = [I18n.t("rumi.ldap.message.synchro") + "<br />"]
    messages << I18n.t("rumi.attributes.group")
    messages << "-- " + I18n.t("rumi.attributes.update") + "#{@results[:group]}" + I18n.t("helpers.number.unit.addresses")
    messages << "-- " + I18n.t("rumi.attributes.delete") + "#{@results[:gdel]}" + I18n.t("helpers.number.unit.addresses") if @results[:gdel] > 0
    messages << "-- " + I18n.t("rumi.attributes.fail") + "#{@results[:gerr]}" + I18n.t("helpers.number.unit.addresses") if @results[:gerr] > 0
    messages << I18n.t("rumi.attributes.user")
    messages << "-- " + I18n.t("rumi.attributes.update") + "#{@results[:user]}" + I18n.t("helpers.number.unit.addresses")
    messages << "-- " + I18n.t("rumi.attributes.delete") + "#{@results[:udel]}" + I18n.t("helpers.number.unit.addresses") if @results[:udel] > 0
    messages << "-- " + I18n.t("rumi.attributes.fail") + "#{@results[:uerr]}" + I18n.t("helpers.number.unit.addresses") if @results[:uerr] > 0
    flash[:notice] = messages.join('<br />').html_safe

    redirect_to(:action => :index)
  end

protected
  def do_synchro(group, parent = nil)
    ## group
    sg                = Sys::Group.find_by(code: group.code) || Sys::Group.new
    sg.code           = group.code
    sg.parent_id      = parent.id
    sg.state        ||= 'enabled'
    sg.web_state    ||= 'public'
    sg.name           = group.name
    sg.name_en        = group.name_en if !group.name_en.blank?
    sg.email          = group.email if !group.email.blank?
    sg.group_s_name   = group.group_s_name
    sg.level_no       = parent.level_no + 1
    #sg.sort_no        = group.sort_no
    sg.ldap         ||= 1
    sg.ldap_version   = @version

    if sg.ldap == 1
      if sg.save(:validate => false)
        @results[:group] += 1
      else
        @results[:gerr] += 1
        return false
      end
    end

    ## users
    if group.users.size > 0
      group.users.each do |user|
        su                   = Sys::User.find_by(code: user.code) || Sys::User.new
        su.code              = user.code
        su.state           ||= 'enabled'
        su.auth_no         ||= 2
        su.name              = user.name
        su.name_en           = user.name_en
        su.email             = user.email
        su.kana              = user.kana
        su.sort_no           = user.sort_no
        su.official_position = user.official_position
        su.assigned_job      = user.assigned_job
        su.group_s_name      = user.group_s_name
        su.ldap            ||= 1
        su.ldap_version      = @version
        su.in_group_id       = sg.id

        if su.ldap == 1
          if su.save
            @results[:user] += 1
          else
            @results[:uerr] += 1
          end
        end
      end
    end

    ## next
    if group.children.size > 0
      group.children.each {|g| do_synchro(g, sg)}
    end
  end

  def create_synchros(entry = nil, group_id = nil)
    if entry.nil?
      Core.ldap.group.children.each do |e|
        create_synchros(e, 0)
      end
      return true
    end

    group = Sys::LdapSynchro.new({
      :parent_id    => group_id,
      :version      => @version,
      :entry_type   => 'group',
      :code         => entry.code,
      :name         => entry.name,
      :name_en      => entry.name_en,
      :email        => entry.email,
      :group_s_name => entry.group_s_name,
    })
    unless group.save
      @results[:error] += 1
      return false
    end
    @results[:group] += 1

    entry.users.each do |e|
      user = Sys::LdapSynchro.new({
        :parent_id         => group.id,
        :version           => @version,
        :entry_type        => 'user',
        :code              => e.uid,
        :name              => e.name,
        :name_en           => e.name_en,
        :email             => e.email,
        :kana              => e.kana,
        :sort_no           => e.sort_no,
        :official_position => e.official_position,
        :assigned_job      => e.assigned_job,
        :group_s_name      => e.group_s_name
      })
      user.save ? @results[:user] += 1 : @results[:error] += 1
    end

    entry.children.each do |e|
      create_synchros(e, group.id)
    end
  end
end
