# coding: utf-8
# === 共有アドレス帳管理権限機能コントローラ
class Sys::Admin::PublicAddressBookRolesController < Sys::Controller::Admin::Base
  include Sys::Controller::Scaffold::Base
  before_action :set_title
  before_action :set_role, only: [:edit, :update]
  before_action :set_before_url, only: [:new, :edit]
  
  def set_before_url
    before_path = Rails.application.routes.recognize_path(request.referrer)
    if ["index", "show"].include?(before_path[:action])
      session[:request_referrer] = request.referrer
    end
  end

  def pre_dispatch
    return error_auth unless Core.user.has_auth?(:manager)
    return redirect_to(request.env['PATH_INFO']) if params[:reset]
  end

  # === 共有アドレス帳管理権限一覧アクション
  def index
    # find_or_create
    @role = Sys::PublicAddressBookRole.first_or_create
  end

  # === 共有アドレス帳管理権限編集アクション
  def edit
  end

  # === 共有アドレス帳管理権限更新アクション
  def update
    @role.attributes = role_params

    _update(@role)
  end

  # 以下、プライベートメソッド
  private
    #ヘッダのタイトルを設定するメソッド
    def set_title
      @title = I18n.t('rumi.mail.title.setting.admin.public_address_book_role')
    end

    # === 共有アドレス帳管理権限をセットするメソッド
    #
    # ==== 引数
    #  * なし
    # ==== 戻り値
    #  http_error(404)
    def set_role
      @role = Sys::PublicAddressBookRole.first_or_create

      http_error(404) if @role.blank?
    end

    def role_params
      params.require(:sys_public_address_book_role).permit!
    end

end
