# encoding: utf-8
module ApplicationHelper
  ## nl2br
  def br(str)
    str.gsub(/\r\n|\r|\n/, '<br />')
  end
  
  ## nl2br and escape
  def hbr(str)
    str = html_escape(str)
    str.gsub(/\r\n|\r|\n/, '<br />')
  end
  
  ## wrap long string
  def text_wrap(text, col = 80, char = " ")
    text = text.gsub("\r\n", "\n")
    text.gsub(/(.{#{col}})(( *$\n?)| +)|(.{#{col}})/) do |match|
      if $3
        "#{$1}#{$2}"
      else
        "#{$1}#{$2}#{$4}#{char}"
      end
    end
  end
  
  ## safe calling
  def safe(alt = nil, &block)
    begin
      yield
    rescue NoMethodError => e
      # nil判定を追加
        alt
    end
  end
  
  ## paginates
  def paginate(items, options = {})
    return '' unless items
    defaults = {
      :params         => p,
      :previous_label => I18n.t("helpers.link.previous_label"),
      :next_label     => I18n.t("helpers.link.next_label"),
      :separator      => '<span class="separator"> | </span' + "\n" + '>'
    }
    if request.mobile? && !request.smart_phone?
      defaults[:page_links]     = false
      defaults[:previous_label] = '&lt;&lt;*' + I18n.t("helpers.button.before")
      defaults[:next_label]     = I18n.t("helpers.button.next") + '#&gt;&gt;'
    end
    links = will_paginate(items, defaults.merge!(options))
    return links if links.blank?
    if Core.request_uri != Core.internal_uri
      links.gsub!(/href="#{URI.encode(Core.internal_uri)}/) do |m|
        m.gsub(/^(href=").*/, '\1' + URI.encode(Core.request_uri))
      end
    end
    if request.mobile? && !request.smart_phone?
      links.gsub!(/<a [^>]*?rel="prev( |")/) {|m| m.gsub(/<a /, '<a accesskey="*" ')}
      links.gsub!(/<a [^>]*?rel="next( |")/) {|m| m.gsub(/<a /, '<a accesskey="#" ')}
    end
    links
  end
  
  ## Number format
  def number_format(num)
    number_to_currency(num, :unit => '', :precision => 0)
  end

  #show tag if condition is true.
  def show_tag_if(tag, cond, options = {}, &block)
    unless cond
      options[:style] ||= ''
      options[:style] += 'display:none;'
    end
    
    content_tag(tag, options, &block)
  end

  def switch_account?
    (Core.user.account != Core.current_user.account)
  end
end