# coding: utf-8
class Gw::EditLinkPiece < ActiveRecord::Base
  establish_connection :enisysgw
  self.table_name = 'gw_edit_link_pieces'

  belongs_to :parent, :class_name => 'Gw::EditLinkPiece', :foreign_key => :parent_id
  has_many :children, :class_name => 'Gw::EditLinkPiece', :foreign_key => :parent_id
  has_many :opened_children, -> {where("published = 'opened' and state != 'deleted'").order('sort_no')}, :class_name => 'Gw::EditLinkPiece', :foreign_key => :parent_id
  belongs_to :css, :class_name => 'Gw::EditLinkPieceCss', :foreign_key => :block_css_id
  belongs_to :icon, :class_name => 'Gw::EditLinkPieceCss', :foreign_key => :block_icon_id

  # === 並び替えのスコープ
  #  並び順の昇順
  # ==== 引数
  #  * なし
  # ==== 戻り値
  #  抽出結果(ActiveRecord::Relation)
  scope :order_sort_no, -> { order("gw_edit_link_pieces.sort_no") }

  # === 階層レベル2のみ抽出するためのスコープ
  #
  # ==== 引数
  #  * なし
  # ==== 戻り値
  #  抽出結果(ActiveRecord::Relation)
  scope :extract_level_no_2, -> { where(level_no: 2) }

  # === 配置場所がポータル ヘッダのみ抽出するためのスコープ
  #
  # ==== 引数
  #  * なし
  # ==== 戻り値
  #  抽出結果(ActiveRecord::Relation)
  scope :extract_enable, lambda {
    where(published: "opened", state: "enabled").includes(
      [:css, :icon,
        opened_children: [:css, :icon, :parent,
          opened_children: [:css, :icon, :parent]
        ]
      ]).order_sort_no
  }

  # === 配置場所がポータル ヘッダのみ抽出するためのスコープ
  #
  # ==== 引数
  #  * なし
  # ==== 戻り値
  #  抽出結果(ActiveRecord::Relation)
  scope :extract_location_header, lambda {
    extract_level_no_2.extract_enable.where(location: 2)
  }

  def has_display_auth?
    if self.display_auth.present?
      begin
        return eval(self.display_auth)
      rescue SyntaxError, ScriptError, StandardError
        return false
      end
    end
    return true
  end

  def link_options
    options = {}

    if self.state == 'disabled' || (self.parent && self.parent.state == 'disabled')
      options[:url] = "#void"
      options[:disabled] = 'grayout'
    else
      if self.class_sso == 2
        options[:url] = "/_admin/gw/link_sso/#{self.id}/redirect_pref_pieces"
      else
        options[:url] = self.link_url
      end
    end

    if self.class_external == 2
      options[:target] = '_blank';
    else
      options[:target] = '_self';
    end

    if self.css && self.css.state == 'enabled'
      options[:css_class] = self.css.css_class
    end

    if self.icon && self.icon.state == 'enabled'
      options[:icon_class] = self.icon.css_class
    end

    if self.icon_path.present? && FileTest.exist?(Rails.root.join("public/#{self.icon_path}"))
      options[:icon_path] = self.icon_path
    end

    options
  end
end
