# encoding: utf-8
class Gw::WebmailMailNode < ActiveRecord::Base
  include Sys::Model::Base
  include Sys::Model::Auth::Free

  validates_presence_of :user_id, :uid, :mailbox

  after_create :check_attack_mail

  def self.find_nodes(boxname, uids = nil)
    query = self.where(user_id: Core.current_user.id)
                .where(mailbox: boxname)
    query = query.where(uid: uids) if uids
    return query
  end

  def self.find_nodes_with_ref(boxname, uids = nil)
    query = self.where(user_id: Core.current_user.id)
                .where(mailbox: boxname)
                .where.not(ref_mailbox: nil)
                .where.not(ref_uid: nil)
    return query
  end

  def self.find_ref_nodes(boxname, uids = nil)
    query = self.where(user_id: Core.current_user.id)
                .where(ref_mailbox: boxname)
    query = query.where(ref_uid: uids) if uids
    return query
  end

  def self.delete_nodes(boxname, uids = nil)
    query = self.where(user_id: Core.current_user.id)
                .where(mailbox: boxname)
    query = query.where(uid: uids) if uids
    return query
  end

  def self.delete_ref_nodes(boxname, uids = nil)
    query = self.where(user_id: Core.current_user.id)
                .where(ref_mailbox: boxname)
    query = query.where(ref_uid: uids) if uids
    query.delete_all
  end

  def readable
    return self.where(user_id: Core.current_user.id)
  end

  def editable?
    return true if Core.current_user.has_auth?(:manager)
    user_id == Core.current_user.id
  end

  def deletable?
    return true if Core.current_user.has_auth?(:manager)
    user_id == Core.current_user.id
  end

  def check_attack_mail
    attack_mail_domains = Sys::AttackMailDomain.select(:domain).map(&:domain)
    attack_mail_extensions = Sys::AttackMailExtension.select(:extension).map(&:extension)
    mail_data = Gw::WebmailMail.find_by_uid(self.uid.to_s, :select => self.mailbox, :conditions => ["UNDELETED"])
    if mail_data.present?
      msgs = mail_data.rfc822 || mail_data.mail.to_s
      if msgs.index("Return-Path")
        msgs = msgs.slice(0, msg.index("\r\n\r\n")) rescue msgs
        msg = msgs.gsub(/\r\n/m, "\n").split("\n")
        domain = ""
        msg.each do |m|
          if m.index("Return-Path")
            address = m.split("@")
            domain = address[address.size-1].to_s.gsub(/(<|>)/, "")
            break
          end
        end
        if domain.present?
          self.attack_mail_domain_flg = true if attack_mail_domains.include?(domain)
        end
      else
        if mail_data.friendly_from_addr.present?
          domain = mail_data.friendly_from_addr.split("@")
          if domain[domain.size-1].present?
            self.attack_mail_domain_flg = true if attack_mail_domains.include?(domain[domain.size-1].to_s.gsub(/(<|>)/, ""))
          end
        end
      end
      if self.attack_mail_domain_flg == nil
        self.attack_mail_domain_flg = false
      end

      if mail_data.has_attachments?
        mail_data.attachments.map(&:name).each do |at|
          filename = at.split(".")
          if filename.present?
            extension = filename[filename.size-1].to_s.gsub(/(<|>)/, "")
            if attack_mail_extensions.include?(extension)
              self.attack_mail_extension_flg = true
              break
            end
          end
        end
      end
      if self.attack_mail_extension_flg == nil
        self.attack_mail_extension_flg = false
      end
    else
      self.attack_mail_domain_flg = false
      self.attack_mail_extension_flg = false
    end
    self.save
  end
end
