# encoding: utf-8
class Gw::WebmailSetting < ActiveRecord::Base
  include Sys::Model::Base
  include Sys::Model::Auth::Free

  class Category

    attr_accessor :name
    attr_accessor :title
    attr_reader :configs

    def initialize(name, title)
      @name = name.to_s
      @title = title
      @configs = []
    end
  end

  validates_presence_of :user_id, :name

  @@config_categories = [
    [I18n.t("rumi.sys.webmail_setting.category.list"), :mail_list],
    [I18n.t("rumi.sys.webmail_setting.category.detail"), :mail_detail],
    [I18n.t("rumi.sys.webmail_setting.category.from"), :mail_form],
    [I18n.t("rumi.sys.webmail_setting.category.pop"), :mail_pop],
    [I18n.t("rumi.sys.webmail_setting.category.label"), :mail_label],
    [Enisys.config.application['webmail.sys_address_menu'], :sys_address],
    [Enisys.config.application['webmail.address_group_menu'], :address],
    [Enisys.config.application['webmail.public_address_book_menu'], :public_address_book]
  ]

  @@config_categorizing = {
    mail_list: [:mails_per_page, :mail_list_subject, :mail_list_from_address, :mail_address_history],
    mail_detail: [:html_mail_view, :mail_attachment_view],
    mail_form: [:mail_from, :sign_position, :mail_encoding],
    mail_pop: [:mail_pop_request],
    mail_label: [:label1, :label2, :label3, :label4, :label5, :label6, :label7, :label8, :label9],
    sys_address: [:sys_address_order],
    address: [:address_order],
    public_address_book: [:public_address_order]
  }

  @@config_names = {
    mails_per_page: I18n.t("rumi.config_settings.mail.setting_name.mail_list_count"),
    mail_list_subject: I18n.t("rumi.config_settings.mail.setting_name.subject"),
    mail_list_from_address: I18n.t("rumi.config_settings.mail.setting_name.from_address"),
    mail_address_history: I18n.t("rumi.config_settings.mail.setting_name.quick_address"),
    html_mail_view: I18n.t("rumi.config_settings.mail.setting_name.mail_view"),
    mail_attachment_view: I18n.t("rumi.config_settings.mail.setting_name.attachment_view"),
    mail_from: I18n.t("rumi.config_settings.mail.setting_name.mail_from"),
    mail_form_size: I18n.t("rumi.config_settings.mail.setting_name.mail_form_size"),
    mail_encoding: I18n.t("rumi.config_settings.mail.setting_name.mail_encoding"),
    sign_position: I18n.t("rumi.config_settings.mail.setting_name.sign_position"),
    sys_address_order: I18n.t("rumi.config_settings.mail.setting_name.address_order"),
    address_order: I18n.t("rumi.config_settings.mail.setting_name.address_order"),
    public_address_order: I18n.t("rumi.config_settings.mail.setting_name.address_order"),
    label1: I18n.t("rumi.config_settings.mail.setting_name.label1"),
    label2: I18n.t("rumi.config_settings.mail.setting_name.label2"),
    label3: I18n.t("rumi.config_settings.mail.setting_name.label3"),
    label4: I18n.t("rumi.config_settings.mail.setting_name.label4"),
    label5: I18n.t("rumi.config_settings.mail.setting_name.label5"),
    label6: I18n.t("rumi.config_settings.mail.setting_name.label6"),
    label7: I18n.t("rumi.config_settings.mail.setting_name.label7"),
    label8: I18n.t("rumi.config_settings.mail.setting_name.label8"),
    label9: I18n.t("rumi.config_settings.mail.setting_name.label9"),
    mail_pop_request: I18n.t("rumi.config_settings.mail.setting_name.request")
  }

  @@config_input_types = {
    mails_per_page: :radio,
    mail_list_subject: :radio,
    mail_list_from_address: :radio,
    mail_address_history: :radio,
    html_mail_view: :radio,
    mail_attachment_view: :radio,
    mail_from: :radio,
    mail_form_size: :radio,
    mail_encoding: :radio,
    sign_position: :radio,
    sys_address_order: :radio,
    address_order: :radio,
    public_address_order: :radio,
    label1: :label_color_field,
    label2: :label_color_field,
    label3: :label_color_field,
    label4: :label_color_field,
    label5: :label_color_field,
    label6: :label_color_field,
    label7: :label_color_field,
    label8: :label_color_field,
    label9: :label_color_field,
    mail_pop_request: :radio
  }

  @@config_options = {
    mails_per_page: [["20" + I18n.t("rumi.config_settings.mail.setting_value.default_count"),''],["30" + I18n.t("rumi.config_settings.mail.setting_value.count"),'30'],["40" + I18n.t("rumi.config_settings.mail.setting_value.count"),'40'],["50" + I18n.t("rumi.config_settings.mail.setting_value.count"),'50']],
    mail_list_subject: [[I18n.t("rumi.config_settings.mail.setting_value.subject_1line") + I18n.t("rumi.config_settings.mail.setting_value.default"), ''], [I18n.t("rumi.config_settings.mail.setting_value.subject_wrap"), 'wrap']],
    mail_list_from_address: [[I18n.t("rumi.config_settings.mail.setting_value.show") + I18n.t("rumi.config_settings.mail.setting_value.default"), ''], [I18n.t("rumi.config_settings.mail.setting_value.not_show"), 'omit_address']],
    mail_address_history: [[I18n.t("rumi.config_settings.mail.setting_value.not_show"), '0'], ["5" + I18n.t("rumi.config_settings.mail.setting_value.count_display"), '5'], ["10" + I18n.t("rumi.config_settings.mail.setting_value.count_display") + I18n.t("rumi.config_settings.mail.setting_value.default"), ''], ["15" + I18n.t("rumi.config_settings.mail.setting_value.count_display"), '15'], ["20" + I18n.t("rumi.config_settings.mail.setting_value.count_display"), '20']],
    html_mail_view: [[I18n.t("rumi.config_settings.mail.setting_value.format_html") + I18n.t("rumi.config_settings.mail.setting_value.default"), ''], [I18n.t("rumi.config_settings.mail.setting_value.format_text"), 'text']],
    mail_attachment_view: [[I18n.t("rumi.config_settings.mail.setting_value.format_thumbnail") + I18n.t("rumi.config_settings.mail.setting_value.default"), ''], [I18n.t("rumi.config_settings.mail.setting_value.format_list"), 'list']],
    mail_from: [[I18n.t("rumi.config_settings.mail.setting_value.from_name") + I18n.t("rumi.config_settings.mail.setting_value.default"),''],[I18n.t("rumi.config_settings.mail.setting_value.from_address"),'only_address']],
    mail_form_size: [[I18n.t("rumi.config_settings.mail.setting_value.size_small"),'small'],[I18n.t("rumi.config_settings.mail.setting_value.size_medium") + I18n.t("rumi.config_settings.mail.setting_value.default"),''],[I18n.t("rumi.config_settings.mail.setting_value.size_large"),'large']],
    mail_encoding: I18n.default_locale.to_s == 'ja' ? [['Unicode (UTF-8)','utf-8'],[I18n.t("rumi.config_settings.mail.setting_value.encode") + "(ISO-2022-JP)",'']] : [['Unicode (UTF-8)',''],[I18n.t("rumi.config_settings.mail.setting_value.encode") + "(ISO-2022-JP)",'iso-2022-jp']] ,
    sign_position: [[I18n.t("rumi.config_settings.mail.setting_value.position_before") + I18n.t("rumi.config_settings.mail.setting_value.default"), ''], [I18n.t("rumi.config_settings.mail.setting_value.position_rear"), 'bottom']],
    sys_address_order: [[I18n.t("rumi.config_settings.mail.setting_value.order_address") + I18n.t("rumi.config_settings.mail.setting_value.default"), ''], [I18n.t("rumi.config_settings.mail.setting_value.order_name"), 'name'], [I18n.t("rumi.config_settings.mail.setting_value.order_kana"), 'kana'], [I18n.t("rumi.config_settings.mail.setting_value.order_position"), 'sort_no']],
    address_order: [[I18n.t("rumi.config_settings.mail.setting_value.order_address") + I18n.t("rumi.config_settings.mail.setting_value.default"), ''], [I18n.t("rumi.config_settings.mail.setting_value.order_kana"), 'kana'], [I18n.t("rumi.config_settings.mail.setting_value.order_sort"), 'sort_no']],
    public_address_order: [[I18n.t("rumi.config_settings.mail.setting_value.order_address") + I18n.t("rumi.config_settings.mail.setting_value.default"), ''], [I18n.t("rumi.config_settings.mail.setting_value.order_kana"), 'kana'], [I18n.t("rumi.config_settings.mail.setting_value.order_sort"), 'sort_no']],
    label1: {id: '1', name: I18n.t("rumi.config_settings.mail.setting_value.label1"),   color: '#ff0000', state: 'enabled'},
    label2: {id: '2', name: I18n.t("rumi.config_settings.mail.setting_value.label2"), color: '#ff007f', state: 'enabled'},
    label3: {id: '3', name: I18n.t("rumi.config_settings.mail.setting_value.label3"),   color: '#4444ff', state: 'enabled'},
    label4: {id: '4', name: I18n.t("rumi.config_settings.mail.setting_value.label4"), color: '#008f00', state: 'enabled'},
    label5: {id: '5', name: I18n.t("rumi.config_settings.mail.setting_value.label5"),     color: '#8f8f00', state: 'enabled'},
    label6: {id: '6', name: I18n.t("rumi.config_settings.mail.setting_value.label6"),  color: '#8f3f00', state: 'enabled'},
    label7: {id: '7', name: I18n.t("rumi.config_settings.mail.setting_value.label7"),  color: '#008f8f', state: 'enabled'},
    label8: {id: '8', name: I18n.t("rumi.config_settings.mail.setting_value.label8"),  color: '#5f005f', state: 'enabled'},
    label9: {id: '9', name: I18n.t("rumi.config_settings.mail.setting_value.label9"),  color: '#777777', state: 'enabled'},
    mail_pop_request: [[I18n.t("rumi.config_settings.mail.setting_value.request") + I18n.t("rumi.config_settings.mail.setting_value.default"), '0'], ["5" + I18n.t("rumi.config_settings.mail.setting_value.minute"), '5'], ["15" + I18n.t("rumi.config_settings.mail.setting_value.minute"), '15'], ["30" + I18n.t("rumi.config_settings.mail.setting_value.minute"), '30']]
  }

  @@config_messages = {}

  @@switch_user_max_count = Enisys.config.application['webmail.switch_user_max_count']
  cattr_accessor :switch_user_max_count

  if @@switch_user_max_count > 0
    @@config_categories << [I18n.t("rumi.config_settings.mail.setting_value.switch_user"), :switch_user]

    @@config_categorizing[:switch_user] = []
    (1..@@switch_user_max_count).each do |i|
      @@config_categorizing[:switch_user] << "switch_user#{i}".intern
    end

    (1..@@switch_user_max_count).each do |i|
      @@config_names["switch_user#{i}".intern] = I18n.t("rumi.config_settings.mail.setting_value.switch_user") + "#{i}"
    end

    (1..@@switch_user_max_count).each do |i|
      @@config_input_types["switch_user#{i}".intern] = :account_password_field
    end
  end

  def self.user_config_categories
    categories = []
    @@config_categories.each do |name, title|
      categories << Category.new(name, title)
    end
    categories
  end

  def self.user_categorized_configs
    #load configs and save to hash
    tmp = {}
    cond = {:user_id => Core.user.id }
    where(cond).each do |conf|
      tmp[conf.name.intern] = conf
    end
    #categorize
    categories = []
    @@config_categories.each do |title, name|
      cat = Category.new(name, title)
      @@config_categorizing[name].each do |conf_name|
        conf = tmp[conf_name] || self.new(cond.merge(:name => conf_name.to_s, :value => ''))
        cat.configs << conf
      end
      categories << cat
    end
    categories
  end

  def self.user_configs
    configs = []
    @@config_names.each do |name, title|
      cond = {user_id: Core.user.id, name: name}
      conf = self.where(cond).first || self.new(cond)
      configs << conf
    end
    configs
  end

  def self.user_config(name)
    cond = {user_id: Core.user.id, name: name}
    query = self.where(cond).first
    query = self.new(cond.merge(value: '')) if query.blank?
    return query
  end

  def self.user_config_value(name, nullif = nil)
    if config = user_config(name)
      return config.value unless config.value.blank?
    end
    return nullif
  end

  def self.user_config_values(names)
    cond = ["user_id = ? and name in (?) ", Core.user.id, names]
    where(cond).inject(HashWithIndifferentAccess.new) do |hash, conf|
      hash[conf.name] = conf.value
      hash
    end
  end

  def config_name
    @@config_names[self.name.intern]
  end

  def options
    @@config_options[self.name.intern]
  end

  def input_type
    @@config_input_types[self.name.intern]
  end

  def message
    @@config_messages[self.name.intern]
  end

  def value_name
    case input_type
    when :radio
      options.each {|name, val| return name if value.to_s == val.to_s } if options
    when :text_field
      return value.blank? ? nil : value
    when :password_field
      return value.blank? ? nil : value.gsub(/./,"*")
    when :account_password_field
      return value.blank? ? nil : JSON.parse(value)['account']
    when :label_color_field
      return value.blank? ? options[:name] : JSON.parse(value)['name']
    end
    value.blank? ? nil : value
  end

  def set_value(params)
    case input_type
    when :account_password_field
      hash = {}
      if !params[:account].blank?
        hash[:account] = params[:account]
        hash[:password] = Util::String::Crypt.encrypt_with_mime(params[:password]) || ''
        self.value = hash.to_json
      else
        self.value = ''
      end
    when :label_color_field
      if !params[:name].blank? && !params[:color].blank?
        self.value = {:id => params[:label_id], :name => params[:name], :color => params[:color], :state => params[:state]}.to_json
      else
        self.value = ''
      end
    else
      self.value = params[:item][:value]
    end
  end

  def readable
    return self.where(user_id: Core.user.id)
  end

  def editable?
    return true if Core.user.has_auth?(:manager)
    user_id == Core.user.id
  end

  def deletable?
    return true if Core.user.has_auth?(:manager)
    user_id == Core.user.id
  end

  def self.switch_user_count
    self.count(conditions: ["user_id = ? AND name LIKE 'switch_user%'", Core.user.id])
  end

  def self.load_switch_user_confs
    names = []
    (1..@@switch_user_max_count).each{|idx| names << "switch_user#{idx}"}
    values = Gw::WebmailSetting.user_config_values(names)

    confs = []
    (1..@@switch_user_max_count).each do |idx|
      value = values["switch_user#{idx}"]
      unless value.blank?
        value = JSON.parse(value)
        confs << {:account => value['account'], :password => value['password']}
      end
    end
    confs
  end

  def self.load_switch_users
    users = [Core.user]
    confs = self.load_switch_user_confs
    confs.each do |conf|
      if user = Sys::User.find_by_code(conf[:account])
        user.password = Util::String::Crypt.decrypt_with_mime(conf[:password]) || ''
        users << user
      end
    end
    users
  end

  def self.load_label_confs
    names = []
    (1..9).each{|idx| names << "label#{idx}"}
    values = Gw::WebmailSetting.user_config_values(names)
    confs = []
    (1..9).each do |idx|
      value = values["label#{idx}"]
      if !value.blank?
        conf = JSON.parse(value) || {}
        conf.symbolize_keys!
        confs << conf if conf[:state] == 'enabled'
      elsif conf = @@config_options["label#{idx}".intern]
        confs << conf if conf[:state] == 'enabled' && !conf[:name].blank?
      end
    end
    confs
  end
end
